#include "lcdbinwindow.h"

#include <qaction.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qapplication.h>
#include <qmessagebox.h>

#include <qstatusbar.h>

#include "maindialogimpl.h" 
#include "imageIcon.h" // pour image0 : icone
#include "XPM/filesave.xpm"
#include "XPM/fileopen.xpm"
#include "XPM/fileprint.xpm"

LCDbinWindow::LCDbinWindow( QWidget* parent, const char* name, WFlags f )
	: QMainWindow( parent, name, f )
{
	// Add your code

	tabwidget = new QTabWidget( this );
	mainDialog = new maindialogImpl(tabwidget);
	tabwidget->addTab(mainDialog, "LCDBin" );
	
	/*QHBox *hbox = new QHBox( tabwidget );
	hbox->setMargin( 5 );
	tabwidget->addTab( hbox, "Clipboard" );
	//tabwidget->addTab( new ListBoxCombo( tabwidget ), "Listboxes/Comboboxes" );*/

	lblClip = new QLabel(this);
	tabwidget->addTab( lblClip, "Clipboard qpixmap");
	
	txtClip = new QTextView(this);
	txtClip->setFont( QFont("Courier New", 8, QFont::Normal));//Helvetica", 12, QFont::Bold));


	tabwidget->addTab( txtClip, "Clipboard text");
	
	setCentralWidget( tabwidget );


	//--- Creation du menu ---
	QPixmap openIcon, saveIcon, printIcon;
	openIcon = QPixmap( fileopen );
	saveIcon = QPixmap( filesave );
	printIcon = QPixmap( fileprint );


    // Creation du menu FILE
	file = new QPopupMenu();
	file->insertItem(openIcon, "Charger BMP",  mainDialog, SLOT(loadBmpClick()), CTRL+Key_O );
	file->insertItem(openIcon, "Charger BIN",  mainDialog, SLOT(loadBinClick()) );
	file->insertSeparator();
	mSaveBmp = file->insertItem(saveIcon, "Sauvegarder BMP",  mainDialog, SLOT(saveBmpClick()) );
	mSaveBin = file->insertItem(saveIcon, "Sauvegarder BIN",  mainDialog, SLOT(saveBinClick()), CTRL+Key_S );
	file->insertSeparator();
	file->insertItem("Quitter",  qApp, SLOT(quit()), CTRL+Key_Q );


	// Creation du menu ABOUT (AIDE)
	about = new QPopupMenu();
	about->insertItem( "&About", this, SLOT(menuAboutMe()), Key_F1 );
	about->insertSeparator();
	about->insertItem( "About &Qt", this, SLOT(menuAboutQt()) );

	// Creation du menu THEME
	theme = new QPopupMenu();
	theme->setCheckable( TRUE );	// #INUTILE
	sMotif = theme->insertItem( "&Motif", this, SLOT(styleMotif()) );
	sMotifPlus = theme->insertItem( "&Motif+", this, SLOT(styleMotifPlus()) );
	sCDE = theme->insertItem( "&CDE", this, SLOT(styleCDE()) );
	sPlatinum = theme->insertItem( "&Platinum", this, SLOT(stylePlatinum()) );
	sWindows = theme->insertItem( "&Windows", this, SLOT(styleWindows()) );
	appFont = QApplication::font();
	

	// Creation du menu EDIT
	edit = new QPopupMenu();
	//edit->insertItem( "Copier Bmp", mainDialog, SLOT(menuCopyBmp()) );
	//edit->insertSeparator();
	edit->insertItem( "Copier Bin", mainDialog, SLOT(menuCopyBin()), CTRL+Key_C );


	// Create a menu bar
	menubar = new QMenuBar(this );
	menubar->setSeparator( QMenuBar::InWindowsStyle );
	menubar->insertItem("&Fichier", file );
	menubar->insertItem("&Edition", edit);
	menubar->insertItem("&Thme", theme);
	menubar->insertItem("&Aide", about);

	menubar->setItemChecked( sPlatinum, TRUE );	// Coche le style Platinum

	statusBar()->message("Sbastien JEFFROY 2003");  // Initial message	

	//setCaption("LCDbin 123");
	QPixmap image0( ( const char** ) image0_data );
   setIcon( image0 );
	mainDialog->setDialogStatusBar(statusBar());

	clipboard = QApplication::clipboard();
	connect(clipboard, SIGNAL(dataChanged()), this, SLOT(clipboardChanged()) );

	
}


void LCDbinWindow::newDocument()
{

/*	maindialogImpl *mainDlg = new maindialogImpl( this, "new", TRUE );
    if ( mainDlg->exec() ) {
        ; // user clicked OK
    }
    else {
        ; // user clicked Cancel
    }
    delete mainDlg;
*/
// TEST
	maindialogImpl *mainDialogNew = new maindialogImpl(tabwidget);
	tabwidget->addTab(mainDialogNew, "LCDBin TEST" );

}




void LCDbinWindow::menuAboutMe()
{
	QMessageBox::about(this, "About", 
				"      Convertisseur Bitmap <-> Binaire       \n"
				"         pour afficheur LCD 122x32           \n"
				"           By Sbastien JEFFROY              \n"
				"         2003 FREEWARE GNU GPL");
}

void LCDbinWindow::menuAboutQt()
{
//    QMessageBox::aboutQt( this, "Qt Themes Example" );
    QMessageBox::aboutQt( this, "LCDbin122x32    Sbastien JEFFROY 2003" );
}


void LCDbinWindow::clipboardChanged()
{
	//QMessageBox::information( this, tr("Message"), tr("Clipboard!") );
	
	// retourne null si vide

	if (clipboard->pixmap().isNull()==FALSE)
	{
		lblClip->setPixmap(clipboard->pixmap()); 
	}

	if (clipboard->text().isNull()==FALSE) 
	{
		txtClip->setText(clipboard->text()); 
	}

/*	if (clipboard->image().isNull==FALSE) 
	{
		QImage *im = new QImage(clipboard->image());
		QPushButton *button = new QPushButton(this);
		QPixmap *pm;
		*pm = *im;
		button->setPixmap(*pm); 
		tabwidget->addTab( button , "Clp qiamge");
	}*/

}


//==========================================================================
//			private slot s
//==========================================================================

// STYLES
void LCDbinWindow::stylePlatinum()
{
    qApp->setStyle(new QPlatinumStyle);
    //QPalette p( QColor(54, 10, 120) );//239, 239, 239 ) );
    //qApp->setPalette(p, TRUE);
    qApp->setFont(appFont, TRUE);
    selectStyleMenu(sPlatinum);
}

void LCDbinWindow::styleWindows()
{
    qApp->setStyle(new QWindowsStyle);
    qApp->setFont(appFont, TRUE);
    selectStyleMenu(sWindows);
}

void LCDbinWindow::styleCDE()
{
    qApp->setStyle(new QCDEStyle(TRUE));
    selectStyleMenu(sCDE );

    QPalette p( QColor( 75, 123, 130 ) );
    p.setColor( QPalette::Active, QColorGroup::Base, QColor( 55, 77, 78 ) );
    p.setColor( QPalette::Inactive, QColorGroup::Base, QColor( 55, 77, 78 ) );
    p.setColor( QPalette::Disabled, QColorGroup::Base, QColor( 55, 77, 78 ) );
    p.setColor( QPalette::Active, QColorGroup::Highlight, Qt::white );
    p.setColor( QPalette::Active, QColorGroup::HighlightedText, QColor( 55, 77, 78 ) );
    p.setColor( QPalette::Inactive, QColorGroup::Highlight, Qt::white );
    p.setColor( QPalette::Inactive, QColorGroup::HighlightedText, QColor( 55, 77, 78 ) );
    p.setColor( QPalette::Disabled, QColorGroup::Highlight, Qt::white );
    p.setColor( QPalette::Disabled, QColorGroup::HighlightedText, QColor( 55, 77, 78 ) );
    p.setColor( QPalette::Active, QColorGroup::Foreground, Qt::white );
    p.setColor( QPalette::Active, QColorGroup::Text, Qt::white );
    p.setColor( QPalette::Active, QColorGroup::ButtonText, Qt::white );
    p.setColor( QPalette::Inactive, QColorGroup::Foreground, Qt::white );
    p.setColor( QPalette::Inactive, QColorGroup::Text, Qt::white );
    p.setColor( QPalette::Inactive, QColorGroup::ButtonText, Qt::white );
    p.setColor( QPalette::Disabled, QColorGroup::Foreground, Qt::lightGray );
    p.setColor( QPalette::Disabled, QColorGroup::Text, Qt::lightGray );
    p.setColor( QPalette::Disabled, QColorGroup::ButtonText, Qt::lightGray );
    qApp->setPalette( p, TRUE );
    qApp->setFont( QFont( "times", appFont.pointSize() ), TRUE );
}

void LCDbinWindow::styleMotif()
{
    qApp->setStyle(new QMotifStyle);
    QPalette p( QColor(192, 192, 192 ));
    qApp->setPalette(p, TRUE);
    qApp->setFont(appFont, TRUE);
    selectStyleMenu(sMotif);
}

void LCDbinWindow::styleMotifPlus()
{
    qApp->setStyle(new QMotifPlusStyle);
    QPalette p( QColor(192, 192, 192 ));
    qApp->setPalette(p, TRUE);
    qApp->setFont(appFont, TRUE);
    selectStyleMenu(sMotifPlus);
}



void LCDbinWindow::selectStyleMenu(int s)
{
   //menubar->setItemChecked( sWood, FALSE );
    //menubar->setItemChecked( sMetal, FALSE );
    menubar->setItemChecked(sPlatinum, FALSE);
    menubar->setItemChecked(sWindows, FALSE);
    menubar->setItemChecked(sCDE, FALSE);
    menubar->setItemChecked(sMotif, FALSE);
    menubar->setItemChecked(sMotifPlus, FALSE);
    menubar->setItemChecked(s, TRUE);
}
