#include "maindialogimpl.h"


/* 
 *  Constructs a maindialogImpl which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
maindialogImpl::maindialogImpl( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : mainDialogBase( parent, name, modal, fl ) 
{
	_color = QColor(255, 255, 255); 
	memset(_data, 0, 488*32);
	cb = QApplication::clipboard();
}


/*  
 *  Destroys the object and frees any allocated resources
 */
maindialogImpl::~maindialogImpl()
{
    // no need to delete child widgets, Qt does it all for us

}

//==========================================================================
//			public slot s
//==========================================================================

void maindialogImpl::loadBinClick()
{
	int i = 0;
	
	QString filename = QFileDialog::getOpenFileName(".", "Binaire (*.bin)", this);
	if (!filename.isEmpty()) {
		QFile fileIn(filename);
		if (fileIn.open( IO_ReadOnly )) {					// index set to 0
			// 122*32/8 = 488
			if ((fileIn.size() <= 488*67) && (fileIn.size()%488==0)) {	// taille multiple de 488
				
				_nbImage = fileIn.size()/488;				// nombre d'images
				LCDmax->display(_nbImage);
				
				while(!fileIn.atEnd()) {						// Chargement du fichier dans data[]
					_data[i++] = fileIn.getch();
				}
				fileIn.close();

				// Vide la liste
				QListViewItem * myChild = ListView1->firstChild();
				while( myChild ) {
					QListViewItem * myChildDel = myChild;
					myChild = myChild->nextSibling();
					ListView1->takeItem(myChildDel);			// supprime l'item
				}

				//	Rempli la liste avec les nouveaux elements
				QString strItem, strImage;
				for(i=0; i<_nbImage; i++) {
					QListViewItem * item = new QListViewItem( ListView1);//, i );
					strItem.sprintf("%2d", i);
					strImage.sprintf("Image %d", i);
					item->setText( 0, tr(strItem) );
					item->setText( 1, tr(strImage) );
				}

				displayBinary(0);									// Affiche l'item 0
				TxtBin->setText(filename);
				drawImage(0);										// Affiche l'image
				TabBitmap->setCurrentPage(1);					// Tab 1 -> Binary
				if (_nbImage==1) {
					BtnPrevious->setEnabled(0);
					BtnNext->setEnabled(0);
				}
				else {
					BtnPrevious->setEnabled(1);
					BtnNext->setEnabled(1);
				}
				statusbar->message(tr("Binaire : %1 charg").arg(filename), 3000);  // 3s	
			}
			else QMessageBox::warning( this, tr("Erreur"), tr("Mauvais Format") );
		}
		else QMessageBox::warning( this, tr("Erreur"), tr("Fichier Introuvable !"));
	}
}


// public slot : chargement d'une image BITMAP ou PNG
// ## MANQUE PREVIEW
// ###### OK 122x32
void maindialogImpl::loadBmpClick()	
{
	QFileDialog fd;
	/*MyPreview *preview = new MyPreview();

	fd.setInfoPreviewEnabled(TRUE);
	fd.setInfoPreview( preview, preview );

	fd.setPreviewMode(QFileDialog::Info);//Contents);
	//fd.setViewMode(QFileDialog::Detail);*/

	/*fd.setContentsPreviewEnabled(TRUE);
	fd.setContentsPreview(preview, preview );*/


	//QString file = QFileDialog::getOpenFileName(".", "Images (*.bmp)", this);// *.gif *.jpg
	//QString file = fd.getOpenFileName(".", "Images (*.bmp)", this);// *.gif *.jpg
	QString file = fd.getOpenFileName(".", "Images (*.bmp *.png)", this);// *.gif *.jpg
	//PNG, BMP, XBM, XPM and PNM
	if (!file.isEmpty()) {
		
		QImageIO imageio;
		QPixmap  pixmap;
		imageio.setFileName(file);							
		
		// ##### FORMAT  fichier bmp seulement !!
		if ( (strcmp(imageio.imageFormat(file), "BMP")) 
			&& (strcmp(imageio.imageFormat(file), "PNG")) ) {
			QString str;
			str.sprintf("Format inconnu %s", imageio.imageFormat(file));

			QMessageBox::warning( this, tr("Erreur"), tr("Format d'image inconnu\n"
										"(Bitmap ou PNG uniquement)"));
			return;
		}

		if ( imageio.read() )  pixmap = imageio.image();  // ok -> convert to pixmap
		
		// teste format : 122 * 32 pixels	         
		if ((pixmap.width()!=122) || (pixmap.height()!=32)) {
			QMessageBox::warning( this, tr("Erreur"), tr("Taille de l'image diffrente\n"
										"de 122 * 32 pixels"));
			return;
		}
		
		this->PixmapBmp->setPixmap(pixmap);				// Affiche l'image (QLabel!)
		this->TxtBmp->setText(file);
		
		this->TabBitmap->setCurrentPage(0);				// Tab0 -> Bitmap
		//QMessageBox::information( this, tr("Message"), tr("Item-Loaded!") );
	}
	statusbar->message(tr("Bitmap : %1 charg").arg(file), 3000);  // 3s	

}

// #### BUG 122x32
// public slot : sauvegarde le QLabel -> QPixmap -> QImage  en binaire
void maindialogImpl::saveBinClick()
{
 	int i = 0;
	
	QString filename = QFileDialog::getSaveFileName(".", "Binaire (*.bin)", this);
	if (!filename.isEmpty()) {
		QFile fileOut(filename);
		if (fileExists(fileOut, filename)==TRUE) return;

		if (fileOut.open( IO_WriteOnly )) {				// index set to 0

			// Decodage Bmp -> bin 			
			QImage im;
			QImage *ptrIm;
			QString str;
			im = *(PixmapBmp->pixmap());	
			ptrIm = &im;
			// ptrIm->pixel(int x, int y); // couleur du pixel (x, y)

			char data[122*32/8];	// 488
			unsigned char color;
			int x, x1, y2, pow = 0, index=0, cx, cy;
			memset(data, 0, 122*32/8);							// remplit le tableau de 0.
			

			//The gray value is calculated using the formula: (r*11 + g*16 + b*5)/32 
			// moitie gauche
			for (x=0; x<4; x++) {				// 4 pages
				for (x1=0; x1<61; x1++) {		// 61 pixels
					pow = 1;
					for(y2=0; y2<8; y2++) {		// 8 pixels
						cy = x*8+y2;
						cx = x1;
						color = (unsigned char)(qGray(qRed(ptrIm->pixel(cx, cy)), 
						qGreen(ptrIm->pixel(cx, cy)), qBlue(ptrIm->pixel(cx, cy))));
							if (color<40) data[index] |= pow;//127	// image 1 bit/couleur => 43
						pow *= 2;
					}
					index++;
				}
			}

			// moitie droite
			for (x=0; x<4; x++) {
				for (x1=61; x1<122; x1++) {
					pow = 1;
					for(y2=0; y2<8; y2++) {
						cy = x*8+y2;
						cx = x1;
						color = (char)(qGray(qRed(ptrIm->pixel(cx, cy)), 
						qGreen(ptrIm->pixel(cx, cy)), qBlue(ptrIm->pixel(cx, cy))));
						if (color<40) data[index] |= pow;//127
						pow *= 2;
					}
					index++;
				}
			}
			fileOut.writeBlock( data, 488);      
			fileOut.close();
			TabBitmap->setCurrentPage(0);
			statusbar->message(tr("Binaire : %1 sauv").arg(filename), 3000);  // 3s	
		}
		else QMessageBox::warning( this, tr("Erreur"), tr("Erreur fichier !") );
	}
}

// public slot : sauvegarde le QLabel -> QPixmap (-> QImage)  en bitmap
void maindialogImpl::saveBmpClick()
{
	// boite de dialogue de sauvegarde
	QString file = QFileDialog::getSaveFileName(".", "Images (*.bmp)", this);
	if (!file.isEmpty()) {
		QFile fileOut(file);	// juste pour tester si le fichier existe
		if (fileExists(fileOut, file)==TRUE) return;

		/*QImageIO imageio;
		QImage   im;
		im = *(PixmapBin->pixmap()); // convert to image couleurs 24 bits!!!
		im.convertDepth(1);	// depth  
		imageio.setImage(im);
		imageio.setFileName(file);
		imageio.setFormat("BMP");
		imageio.convertDepth(1);	// depth  
		imageio.write(); */
		PixmapBin->pixmap()->save (file, "BMP");
		statusbar->message(tr("Bitmap : %1 sauv").arg(file), 3000);  // 3s	
	}
}


/* 
 * public slot
 */
void maindialogImpl::tabChanged()
{
	switch(TabBitmap->currentPageIndex()) {
		case 0:	btnSaveBmp->setEnabled(0);
					btnSaveBin->setEnabled(1);
/*					menubar->setItemEnabled( mSaveBmp, FALSE );
					menubar->setItemEnabled( mSaveBin, TRUE );*/
					break;
		case 1:	btnSaveBmp->setEnabled(1);
					btnSaveBin->setEnabled(0);
/*					menubar->setItemEnabled( mSaveBmp, TRUE );
					menubar->setItemEnabled( mSaveBin, FALSE  );*/
					break;
		case 2:	btnSaveBmp->setEnabled(1);
					btnSaveBin->setEnabled(0);
/*					menubar->setItemEnabled( mSaveBmp, TRUE );
					menubar->setItemEnabled( mSaveBin, FALSE  );*/
					break;
	}
}

/* 
 * public slot
 */
void maindialogImpl::itemClick(int button,QListViewItem* item,const QPoint& pt,int j)
{
	int pos = ListView1->itemPos(item);
	/*QString str;
	str.sprintf("button %d ?? : %d itemPos %d", button, j, pos);
	QMessageBox::information( this, tr( "Message" ), tr(str) );//"HELLO CLK"));//*/
	
	switch(button)
	{
		case 1:	// bouton gauche
					displayBinary(pos/16);	// /16 ???
					drawImage(pos/16);
					break;
		case 2:
					break;
	}
}


// Image precedante
void maindialogImpl::previousClick()
{
	if (LCDindex->intValue() > 0) {
		drawImage(LCDindex->intValue()-1);
		ListView1->clearSelection();					// deselectionne l'item de la liste
		//ListView1->setSelected(QListViewItem *, bool);
		displayBinary(LCDindex->intValue());
	}
}

// Image suivante
void maindialogImpl::nextClick()
{
	if (LCDindex->intValue() < _nbImage-1) {
		drawImage(LCDindex->intValue()+1);
		ListView1->clearSelection(); 					// deselectionne l'item de la liste
		displayBinary(LCDindex->intValue());
	}
}
 
// Couleur de fond Image binaire
void maindialogImpl::colorClick()
{
	QColor color;
	color = QColorDialog::getColor(BtnColor->backgroundColor());
	
	if (color.isValid()) {								// si OK
		_color = color;

		QPalette pal;
		pal = BtnColor->palette();
		QColorGroup cg;
		cg.setColor(QColorGroup::Button, color);
		pal.setActive(cg);
		BtnColor->setPalette(pal);						// change la couleur du bouton

		drawImage(LCDindex->intValue());				// refresh image binaire
		statusbar->message(tr("Couleur %1 %2 %3").arg(color.red()).arg(color.green()).arg(color.blue()), 3000);  // 3s	
	}
}


void maindialogImpl::exportChanged(int indice)
{
	 switch(indice) {
		 case 0:	// Assembleur pour PIC
					LineComment->setText("; ");		
					LineDebutTab->setText("image:");
					LineFinTab->setText("");
					LineDebutLigne->setText("dt $tab");
					LineDebutHexa->setText("0x");
					LineSeparateur->setText(", ");
					break;

		 case 1:	// 'C' pour PIC
					LineComment->setText("// ");		
					LineDebutTab->setText("const unsigned char image[488] = {");
					LineFinTab->setText("};");
					LineDebutLigne->setText("$tab");
					LineDebutHexa->setText("0x");
					LineSeparateur->setText(", ");
					break;

		 case 2:	// Personnalise
					LineComment->setText("' ");
					LineDebutTab->setText("Dim image()=");
					LineFinTab->setText("");
					LineDebutLigne->setText("$tab$tab");
					LineDebutHexa->setText("&H");
					LineSeparateur->setText(", ");
					break;

	 }
}

void maindialogImpl::exportClick()
{
 	int i = 0;
	QString filename;
	if (!BtnPICasm->isChecked()) filename = QFileDialog::getSaveFileName(".", "Source c (*.c)", this);
	else filename = QFileDialog::getSaveFileName(".", "Source assembler (*.asm)", this);
	if (!filename.isEmpty()) {
		QFile fileOut(filename);
		if (fileExists(fileOut, filename)==TRUE) return;

		if (fileOut.open( IO_WriteOnly )) {				// index set to 0

			QString str;
			exportAscii(str);									// remplit la string

			fileOut.writeBlock( str, qstrlen(str) ); 			
			fileOut.close();
			///TabBitmap->setCurrentPage(0);
			statusbar->message(tr("%1 export").arg(filename), 3000);  // 3s	
		}
		else QMessageBox::warning( this, tr("Erreur"), tr("Erreur fichier !") );
	}
	else {
	   //QMessageBox::information( this, tr("Message"), tr("Item-Not Loaded!") );
	}
}


// public slot : copie le PixmapBmp vers le presse papier
void maindialogImpl::menuCopyBmp()
{
	QImage im;
	QString str;
	im = *(PixmapBmp->pixmap());	
	cb->setData(new QImageDrag(im));
	statusbar->message(tr("%1 Image bitmap copie dans le presse papier").arg(im.depth()), 3000);  // 3s	
}

// public slot : copie le PixmapBin vers le presse papier
void maindialogImpl::menuCopyBin()
{
	QImage im;
	QString str;
	im = *(PixmapBin->pixmap());	
	cb->setData(new QImageDrag(im));
	statusbar->message(tr("Image binaire copie dans le presse papier"), 3000);  // 3s	
}

// public slot : copie l'export ASCII vers le presse papier
void maindialogImpl::exportCopieClick()
{
	QString str;
	exportAscii(str);

	cb->setText(str);					// copie vers le presse papier
	statusbar->message(tr("Export ASCII copi dans le presse papier"), 3000);  // 3s	
}


/************ Methodes publiques *************/

// Teste si un fichier existe -> si oui "voulez-vous le remplacer ?" -> OK -> FALSE
bool maindialogImpl::fileExists(QFile fileOut, QString file)
{
	if (fileOut.exists(file)) {
		QMessageBox mb("Enregister Binaire sous", 
						tr("%1 existe dj\nVoulez-vous le remplacer ?").arg(file),
						QMessageBox::NoIcon,
						QMessageBox::Yes | QMessageBox::Default,
						QMessageBox::No,
						QMessageBox::Cancel | QMessageBox::Escape );
		mb.setButtonText(QMessageBox::Yes,  "Oui");
		mb.setButtonText(QMessageBox::No, "Non");
		mb.setButtonText(QMessageBox::Cancel, "Annuler");
		if ( mb.exec() != QMessageBox::Yes ) return TRUE;
	}
	return FALSE;
}

/************ Methodes privees *************/

void maindialogImpl::exportAscii(QString &str)
{
 	int i = 0;

	QString strTmp, comment, debutTab, finTab, 
		debutLigne, debutHexa, separateur; 

	// Convertion des "$tab" en tabulations 
	comment = LineComment->text();
	comment.replace(QRegExp("$tab"), "\t");

	debutTab = LineDebutTab->text();
	debutTab.replace(QRegExp("$tab"), "\t");

	finTab = LineFinTab->text();
	finTab.replace(QRegExp("$tab"), "\t");
	
	debutLigne = LineDebutLigne->text();
	debutLigne.replace(QRegExp("$tab"), "\t");

	debutHexa = LineDebutHexa->text();
	debutHexa.replace(QRegExp("$tab"), "\t");
	
	separateur = LineSeparateur->text();
	separateur.replace(QRegExp("$tab"), "\t");

	int index = 0;
	str += comment; //LineComment->text();
	str += "**********************************\n";
	str += comment;
	str += "*     Exporte par LCDbin122x32   *\n";
	str += comment;
	str += "*   (c)2003 Sebastien JEFFROY    *\n";
	str += comment;
	str += "**********************************\n\n";
	str += debutTab;
	//str += "\n" + debutLigne;

	do{
		//writeBlock limite a 2098 octets
		for(int k=0; k < LCDNumber3->intValue() ; k++) {
			if (index % LCDNumber3->intValue() == 0) {	// +1??
				str += "\n" + debutLigne;
			}
			strTmp.sprintf("%02X", _data[index] & 0xFF); 
			str += debutHexa + strTmp;
			if (index!=487) {
				if ((BtnPICasm->isChecked()) && ((index+1) % LCDNumber3->intValue() == 0)) {
					// Assembleur pour PIC => pas de caractere de separation en fin de ligne
				}
				else str += separateur;
			}
			index++;
			if (index==488) break;//???
		}
	}while(index<488);
	str += "\n" + finTab + "\n\n";
}

void maindialogImpl::setDialogStatusBar(QStatusBar *status)
{
	statusbar = status;
}


// Affichie l'image (Bin)
void maindialogImpl::drawImage(int i)
{
	char *ptrData = &_data[488*i];
	// IMAGE                width,height, depth, numColors=0, Endian bitOrder=IgnoreEndian 
	QImage *image = new QImage (122, 32, 8, 2, QImage::IgnoreEndian);
	
	// Couleurs
	// Pixel ON : NOIR
	image->setColor( 0, qRgb(0, 0,0) );	
	// Pixel OFF : Vert
	image->setColor( 1, _color.rgb());//qRgb(255,255,255));	//0, 255, 0) );		
	
	// set 8 bit pixel at (x,y) to value yellow (in color table)
	//*(image->scanLine(y) + x) = 1;	// 1:couleur

	int x, x1, y2, pow = 0, index=0;

	// moitie gauche
	for (x=0; x<4; x++) {					// 4 pages
		for (x1=0; x1<61; x1++) {			// 62 col
			pow = 1;
			for(y2=0; y2<8; y2++) {			// 8 pixels
				if (ptrData[index] & pow) *(image->scanLine(x*8 + y2) + x1) = 0;
				else *(image->scanLine(x*8 + y2) + x1) = 1;
				pow *= 2;
			}
			index++;
		}
	}

	// moitie droite
	for (x=0; x<4; x++) {					// 4 pages
		for (x1=61; x1<122; x1++) {		// 
			pow = 1;
			for(y2=0; y2<8; y2++) {			// 8 pixels
				if (ptrData[index] & pow) *(image->scanLine(x*8 + y2) + x1) = 0;
				else *(image->scanLine(x*8 + y2) + x1) = 1;
				pow *= 2;
			}
			index++;
		}
	}

	image->convertDepth(8);								//1); // depth 8->1 //####UTILE?
	QPixmap pixmap;
	pixmap = *image;										// copie l'image dans le pixmap
	PixmapBin->setPixmap(pixmap);						// affiche le pixmap dans le QLabel
	PixmapBinMini->setPixmap(pixmap);
	PixmapBinMini2->setPixmap(pixmap);
	delete image;
	LCDindex->display(i);
}

// Affiche les donnees (Bin) en hexa
void maindialogImpl::displayBinary(int i)
{
	char buf[10];
	QString str;
	for(int j=i*488; j<i*488+488; j++) {
		sprintf(buf, "%02X,", _data[j]&0xFF);
		if ((j%488)%16==0) str += '\n';						// retour a la ligne tous les 16
		str += buf;
	}
	TextView1->setText(str);
}


///// EOF //////
